#!/usr/bin/perl

use strict;
use warnings;
use feature ':5.10';

use Text::CSV::Slurp;
use File::Find;

use lib "$ENV{HOME}/src/manalink/gg/magic_updater";
use Manalink::CSV;

my $lf = "\r\n";

my $cardlist_txt = "$ENV{HOME}/pristine/gg/cardlist.txt";

-f $cardlist_txt or die "$cardlist_txt doesn't exist";

my %shandalar;
open(CARDLIST, '<', $cardlist_txt) or die "Couldn't open $cardlist_txt: $!";
while (<CARDLIST>)
{
  s/[\n\r]+$//;
  /^[0-9]+ cards\.$/ and next;
  /^\.([0-9]+)\t(.*)$/ or die qq@Couldn't parse line in $cardlist_txt: "$_"@;
  $shandalar{$1} = $2;
}
close CARDLIST or die "Couldn't close $cardlist_txt: $!";

my $manalink = Text::CSV::Slurp->load(file => "$ENV{HOME}/src/manalink/gg/magic_updater/Manalink.csv", sep_char=>';');

my %csv;
foreach my $ml (@$manalink)
  {
    my $id = $ml->{ID};
    next if ($id eq '#');
    $id =~ /^[0-9]+$/ or die "Couldn't parse ID #$id";

    my $name = decode($ml->{'Full Name'});
    next if $name =~ /^(Empty|None)$/;
    if (exists $csv{$name})
      {
	my $prev = $csv{$name}->{ID};
	die "CSV collision at #$id: \"$name\": previously saw <<$prev>>, now have <<$ml->{ID}";
      }

    $csv{$name} = $ml;
  }

my @files;
say "Searching for files";
find({
      wanted=>sub { -f "$_" && /\.dck$/ and push @files, $_; },
      no_chdir=>1
     }, '.');
say "Found ", scalar @files;

open(SHOK, '>', 'shandalar_ok.txt') or die "Couldn't open shandalar_ok.txt: $!";
open(MLOK, '>', 'manalink_ok.txt') or die "Couldn't open manalink_ok.txt: $!";
open(BOTHOK, '>', 'both_ok.txt') or die "Couldn't open both_ok.txt: $!";
open(NOTOK, '>', 'neither_ok.txt') or die "Couldn't open neither_ok.txt: $!";

foreach my $f (@files)
{
  say "\n", $f;
  open(FIL, '<', $f) or die "Couldn't open $f: $!";
  my @fil;
  while (<FIL>)
    {
      s/[\n\r]+$//;
      push @fil, $_;
    }
  close(FIL);

  my $basename = $f;
  $basename =~ s/\.dck$//;
  $basename =~ s,.*/,,g;
  my ($author, $deckname, $format, $number);
  if ($basename =~ /^(.+?)\s*-\s+(.+?)\s*\(([A-Z][a-z]+), #([0-9]+)\)$/)
    {
      $author = $1;
      $deckname = $2;
      $format = $3;
      $number = $4;
    }
  elsif ($basename =~ /^\s*(.+?)\s*\(([A-Z][a-z]+), #([0-9]+)\)$/)
    {
      $author = "";
      $deckname = $1;
      $format = $2;
      $number = $3;
    }
  else
    {
      die "$f: Couldn't parse author/deckname/format/number from \"$basename\"";
    }
  my $url = "http://www.mtgdecks.net/decks/view/$number";

  my $comment = "";

  my $l = shift @fil;
  while ($l =~ m,^//(.*),)
    {
      $comment =~ /./ and $comment .= ' - ';
      $comment .= $1;
      $l = shift @fil;
    }
  $comment =~ /./ and $comment .= ' - ';
  $comment .= $url;

  my $metadata = $l;
  $metadata eq '[metadata]' or die "$f: no [metadata]";

  my $name = shift @fil;
  $name =~ /^Name=(.*)/ or die "$f: no Name=";
  $name = $1;
  $name =~ /^\Q$basename\E( \[!\])?/ or die "$f: Name doesn't match filename\n\"$name\" !+ \"$f\"";

  my $general = shift @fil;
  $general eq '[general]' or die "$f: no [general]";

  my $Constructed = shift @fil;
  $Constructed eq 'Constructed' or die "$f: no Constructed";

  my $main = shift @fil;
  $main eq '[Main]' or die "$f: no [Main]";

  my $sideboard = 0;
  my $commander = 0;
  my $cardlist = "";
  my $shandalar_ok = 1;
  my $manalink_ok = 1;
  while (scalar @fil)
    {
      my $line = shift @fil;
      $line =~ /^\s*$/ and next;

      if ($line eq '[Sideboard]')
	{
	  $sideboard == 1 and die "$f: multiple [Sideboard]";
	  $sideboard = 1;
	  next;
	}

      if ($line eq '[Commander]')
	{
	  $commander = 1;
	  $sideboard = 0;
	  $shandalar_ok = 0;
	  $cardlist .= ".14675\t1\t* Elder Dragon Highlander$lf";
	  next;
	}

      $line =~ m,^\s*(//\s*)?([1-9][0-9]*)\s+(.*?)\s*$, or die "$f: Couldn't parse \"$line\"";

      $sideboard and next;

      my $num = $2;
      my $card = $3;
      if ($commander)
	{
	  $num *= 2;
	}

      if (exists $csv{$card})
	{
	  my $c = $csv{$card};
	  $cardlist .= '.' . $c->{ID} . "\t" . $num . "\t" . $c->{'Full Name'} . $lf;
	  if (!$c->{'CODED CARD'})
	    {
	      $manalink_ok = 0;
	      say "$f: \"$card\" not Manalink ok";
	    }
	  if (!exists $shandalar{$c->{ID}})
	    {
	      $shandalar_ok = 0;
	      say "$f: \"$card\" not Shandalar ok";
	    }
	}
      else
	{
	  die "$f: Unknown card \"$card\"";
	}
    }
  if ($manalink_ok)
    {
      if ($shandalar_ok)
	{
	  say BOTHOK $f;
	}
      else
	{
	  say MLOK $f;
	}
    }
  else
    {
      if ($shandalar_ok)
	{
	  say SHOK $f;
	}
      else
	{
	  say NOTOK $f;
	}
    }

  if (1)
    {
      my $outf = $f;
      $outf =~ s/\.dck$/.dec/ or die "$f: Couldn't construct outf";

      open(OUT, '>', $outf) or die "Couldn't open $outf: $!";
      print OUT ';', $basename, $lf;	# must match filename.dck
      print OUT ';', $deckname, $lf;	# description
      print OUT ';', $author, $lf;	# author
      print OUT ';User E-Mail', $lf;	# email
      print OUT ';08/03/2016', $lf;	# creation date
      print OUT ';1', $lf;		# revision
      print OUT ';4th Edition', $lf;	# edition
      print OUT ';', $comment, $lf;	# comments
      print OUT $lf;
      print OUT $cardlist;
      close OUT;
    }
}

close MLOK;
close SHOK;
close BOTHOK;
close NOTOK;
